"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
var tslib_1 = require("tslib");
var bn_js_1 = tslib_1.__importDefault(require("bn.js"));
var contract_1 = require("../contract");
var address_1 = require("../address");
var transfer_gateway_pb_1 = require("../proto/transfer_gateway_pb");
var big_uint_1 = require("../big-uint");
var crypto_utils_1 = require("../crypto-utils");
var TransferGateway = /** @class */ (function (_super) {
    tslib_1.__extends(TransferGateway, _super);
    function TransferGateway(params) {
        var _this = _super.call(this, params) || this;
        _this.on(contract_1.Contract.EVENT, function (event) {
            if (!event.topics || event.topics.length === 0) {
                return;
            }
            if (event.topics[0] === TransferGateway.tokenWithdrawalSignedEventTopic) {
                var eventData = transfer_gateway_pb_1.TransferGatewayTokenWithdrawalSigned.deserializeBinary(crypto_utils_1.B64ToUint8Array(event.data));
                var tokenId = void 0, tokenAmount = void 0, value = void 0;
                var tokenKind = eventData.getTokenKind();
                switch (tokenKind) {
                    case transfer_gateway_pb_1.TransferGatewayTokenKind.ERC721:
                        tokenId = big_uint_1.unmarshalBigUIntPB(eventData.getTokenId());
                        value = tokenId;
                        break;
                    case transfer_gateway_pb_1.TransferGatewayTokenKind.ERC721X:
                        tokenId = big_uint_1.unmarshalBigUIntPB(eventData.getTokenId());
                    // fallthrough
                    default:
                        tokenAmount = big_uint_1.unmarshalBigUIntPB(eventData.getTokenAmount());
                        value = tokenAmount;
                        break;
                }
                _this.emit(TransferGateway.EVENT_TOKEN_WITHDRAWAL, {
                    tokenOwner: address_1.Address.UmarshalPB(eventData.getTokenOwner()),
                    tokenContract: address_1.Address.UmarshalPB(eventData.getTokenContract()),
                    tokenKind: tokenKind,
                    tokenId: tokenId,
                    tokenAmount: tokenAmount,
                    sig: eventData.getSig_asU8(),
                    value: value
                });
            }
            else if (event.topics[0] === TransferGateway.contractMappingConfirmedEventTopic) {
                var contractMappingConfirmed = transfer_gateway_pb_1.TransferGatewayContractMappingConfirmed.deserializeBinary(crypto_utils_1.B64ToUint8Array(event.data));
                _this.emit(TransferGateway.EVENT_CONTRACT_MAPPING_CONFIRMED, {
                    foreignContract: address_1.Address.UmarshalPB(contractMappingConfirmed.getForeignContract()),
                    localContract: address_1.Address.UmarshalPB(contractMappingConfirmed.getLocalContract())
                });
            }
        });
        return _this;
    }
    TransferGateway.createAsync = function (client, callerAddr) {
        return tslib_1.__awaiter(this, void 0, void 0, function () {
            var contractAddr;
            return tslib_1.__generator(this, function (_a) {
                switch (_a.label) {
                    case 0: return [4 /*yield*/, client.getContractAddressAsync('gateway')];
                    case 1:
                        contractAddr = _a.sent();
                        if (!contractAddr) {
                            throw Error('Failed to resolve contract address for TransferGateway');
                        }
                        return [2 /*return*/, new TransferGateway({ contractAddr: contractAddr, callerAddr: callerAddr, client: client })];
                }
            });
        });
    };
    TransferGateway.prototype.addContractMappingAsync = function (params) {
        var foreignContract = params.foreignContract, localContract = params.localContract, foreignContractCreatorSig = params.foreignContractCreatorSig, foreignContractCreatorTxHash = params.foreignContractCreatorTxHash;
        var mappingContractRequest = new transfer_gateway_pb_1.TransferGatewayAddContractMappingRequest();
        mappingContractRequest.setForeignContract(foreignContract.MarshalPB());
        mappingContractRequest.setLocalContract(localContract.MarshalPB());
        mappingContractRequest.setForeignContractCreatorSig(foreignContractCreatorSig);
        mappingContractRequest.setForeignContractTxHash(foreignContractCreatorTxHash);
        return this.callAsync('AddContractMapping', mappingContractRequest);
    };
    TransferGateway.prototype.withdrawERC721Async = function (tokenId, tokenContract) {
        var req = new transfer_gateway_pb_1.TransferGatewayWithdrawTokenRequest();
        req.setTokenKind(transfer_gateway_pb_1.TransferGatewayTokenKind.ERC721);
        req.setTokenId(big_uint_1.marshalBigUIntPB(tokenId));
        req.setTokenContract(tokenContract.MarshalPB());
        return this.callAsync('WithdrawToken', req);
    };
    TransferGateway.prototype.withdrawERC721XAsync = function (tokenId, amount, tokenContract) {
        var req = new transfer_gateway_pb_1.TransferGatewayWithdrawTokenRequest();
        req.setTokenKind(transfer_gateway_pb_1.TransferGatewayTokenKind.ERC721X);
        req.setTokenId(big_uint_1.marshalBigUIntPB(tokenId));
        req.setTokenAmount(big_uint_1.marshalBigUIntPB(amount));
        req.setTokenContract(tokenContract.MarshalPB());
        return this.callAsync('WithdrawToken', req);
    };
    TransferGateway.prototype.withdrawERC20Async = function (amount, tokenContract) {
        var req = new transfer_gateway_pb_1.TransferGatewayWithdrawTokenRequest();
        req.setTokenKind(transfer_gateway_pb_1.TransferGatewayTokenKind.ERC20);
        req.setTokenAmount(big_uint_1.marshalBigUIntPB(amount));
        req.setTokenContract(tokenContract.MarshalPB());
        return this.callAsync('WithdrawToken', req);
    };
    TransferGateway.prototype.withdrawETHAsync = function (amount, mainNetGateway) {
        var tgWithdrawETHReq = new transfer_gateway_pb_1.TransferGatewayWithdrawETHRequest();
        tgWithdrawETHReq.setAmount(big_uint_1.marshalBigUIntPB(amount));
        tgWithdrawETHReq.setMainnetGateway(mainNetGateway.MarshalPB());
        return this.callAsync('WithdrawETH', tgWithdrawETHReq);
    };
    TransferGateway.prototype.withdrawalReceiptAsync = function (owner) {
        return tslib_1.__awaiter(this, void 0, void 0, function () {
            var tgWithdrawReceiptReq, result, receipt, tokenId, tokenAmount, value, tokenKind;
            return tslib_1.__generator(this, function (_a) {
                switch (_a.label) {
                    case 0:
                        tgWithdrawReceiptReq = new transfer_gateway_pb_1.TransferGatewayWithdrawalReceiptRequest();
                        tgWithdrawReceiptReq.setOwner(owner.MarshalPB());
                        return [4 /*yield*/, this.staticCallAsync('WithdrawalReceipt', tgWithdrawReceiptReq, new transfer_gateway_pb_1.TransferGatewayWithdrawalReceiptResponse())];
                    case 1:
                        result = _a.sent();
                        receipt = result.getReceipt();
                        if (receipt) {
                            tokenId = void 0, tokenAmount = void 0, value = void 0;
                            tokenKind = receipt.getTokenKind();
                            switch (tokenKind) {
                                case transfer_gateway_pb_1.TransferGatewayTokenKind.ERC721:
                                    tokenId = big_uint_1.unmarshalBigUIntPB(receipt.getTokenId());
                                    value = tokenId;
                                    break;
                                case transfer_gateway_pb_1.TransferGatewayTokenKind.ERC721X:
                                    tokenId = big_uint_1.unmarshalBigUIntPB(receipt.getTokenId());
                                // fallthrough
                                default:
                                    tokenAmount = big_uint_1.unmarshalBigUIntPB(receipt.getTokenAmount());
                                    value = tokenAmount;
                                    break;
                            }
                            return [2 /*return*/, {
                                    tokenOwner: address_1.Address.UmarshalPB(receipt.getTokenOwner()),
                                    tokenContract: address_1.Address.UmarshalPB(receipt.getTokenContract()),
                                    tokenKind: tokenKind,
                                    tokenId: tokenId,
                                    tokenAmount: tokenAmount,
                                    withdrawalNonce: new bn_js_1.default(receipt.getWithdrawalNonce()),
                                    oracleSignature: receipt.getOracleSignature_asU8(),
                                    value: value
                                }];
                        }
                        return [2 /*return*/, null];
                }
            });
        });
    };
    // LoomJS user events
    TransferGateway.EVENT_TOKEN_WITHDRAWAL = 'event_token_withdrawal';
    TransferGateway.EVENT_CONTRACT_MAPPING_CONFIRMED = 'event_contract_mapping_confirmed';
    // Events from Loomchain
    TransferGateway.tokenWithdrawalSignedEventTopic = 'event:TokenWithdrawalSigned';
    TransferGateway.contractMappingConfirmedEventTopic = 'event:ContractMappingConfirmed';
    return TransferGateway;
}(contract_1.Contract));
exports.TransferGateway = TransferGateway;
//# sourceMappingURL=transfer-gateway.js.map